/* Copyright (C) 2005 Juergen "George" Sawinski
 * All rights reserved.
 * Licensed under the LGPL v2.1 or later,
 * see accompanying COPYING.LIB file.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "tar.h"

#ifdef USE_GZIP
#include <zlib.h>
#endif

#ifdef USE_BZIP2
#include <bzlib.h>
#endif

int luatar_filemagic(const char *fname)
{
  int fd;
  unsigned char buf[16] = {0};

  dbg("luatar_filemagic(%s)\n", fname);

  /* read first 16 bytes, ignore errors */
  fd = open(fname, O_RDONLY);
  if (fd < 0) return -1;
  read(fd, buf, 16);
  close(fd);

  /* UNIX COMPRESS */
  if ((buf[0] == 31) && (buf[1] == 157)) return -1 /*LUATAR_TYPE_COMPRESS*/;

  /* GZIP */
  if (buf[0] == 31) {
    if ((buf[1] == 139) 
	|| (buf[1] == 30)) return LUATAR_TYPE_GZIP;
  }

  /* BZIP2 */
  if ((buf[0] == 'B') && (buf[1] == 'Z')) return LUATAR_TYPE_BZIP2;

#define bufcmp(str) !strncmp(buf, str, sizeof(str)-1)
  
  /* LZO */
  if (bufcmp("\x89\x4c\x5a\x4f\x00\x0d\x0a\x1a\x0a")) return -1 /*LUATAR_TYPE_LZO*/;

  /* 7Z */
  if (bufcmp("7z\274\257\047\034")) return -1 /*LUATAR_TYPE_7Z*/;

  /* TAR */
  fd = open(fname, O_RDONLY);
  lseek(fd, 257, SEEK_SET);
  read(fd, buf, 16);
  close(fd);

  if (bufcmp("ustar")) return LUATAR_TYPE_PLAIN;

  return -1;
}

int luatar_filetype(const char *fname)
{
  const char *ext = strrchr(fname, '.');

  int i;
  static struct { int type; const char *suffix; } exts[] = {
    { LUATAR_TYPE_PLAIN, "tar" },
    { LUATAR_TYPE_GZIP,  "gz" },
    { LUATAR_TYPE_BZIP2, "bz2" },
    { LUATAR_TYPE_ZIP,   "Z" },

    { LUATAR_TYPE_GZIP,  "tgz" },
    { LUATAR_TYPE_BZIP2, "tbz2" },
    { LUATAR_TYPE_BZIP2, "tbz" },

    { -1, NULL },
  };

  if (!ext) ext = fname;
  else ext++;

  dbg("luatar_filetype(%s)\n", fname);
  
  for(i=0; exts[i].suffix; i++) {
    if (!strcmp(exts[i].suffix, ext)) {
      dbg("  -> %d\n", exts[i].type);

      return exts[i].type;
    }
  }  

  return -1;
}

int aux_luatar_open(tar_ud *ud, const char *fname, int type, int flags)
{
  int fd;
  void *F = NULL;
  tartype_t *ttype = NULL;

  dbg("aux_luatar_open(%p,%s,%d,%08X)\n", ud, fname, type, flags);

  /* open file */
  fd = open(fname, O_RDONLY);
  if (fd < 0) return 1;

  /* prepare opening file */
  switch (type) {
  case LUATAR_TYPE_PLAIN:
    break;
#ifdef USE_GZIP
  case LUATAR_TYPE_ZIP:
  case LUATAR_TYPE_GZIP: 
    if (!(F = gzdopen(fd, "rb"))) return 1;
    ttype = &c_gzip;
    break;
#endif
#ifdef USE_BZIP2
  case LUATAR_TYPE_BZIP2:
    if (!(F = BZ2_bzdopen(fd, "rb"))) return 1;
    ttype = &c_bzip2;
    break;
#endif
  default:
    errno = ENOSYS;
    return 1;
  }

  /* add stream<->fd mapping */
  if (F)
    luatar_cstream_add(fd, F);

  /* adjust ud */
  ud->read_only   = 1;
  ud->stream_type = type;
  ud->status      = LUATAR_STATUS_INIT;

  /* open tar */
  if (
      tar_fdopen(&(ud->file), fd, /*FIXME why is this (char *) ????*/ (char *) fname, ttype, O_RDONLY, 0, flags)
      ) {
    ud->status = LUATAR_STATUS_ERROR;
    return 1;
  }

  return 0;
}

int aux_luatar_new(tar_ud *ud, const char *fname, int type, int compression, int flags, int mode)
{
  int fd;
  void *F = NULL;
  char zmode[6] = { 'w', 'b', '9', 0, 0, 0 };
  tartype_t *ttype = NULL;

  dbg("aux_luatar_new(%p,%s,%d,%08X,%04o)\n", ud, fname, type, flags, mode);

  /* open file */
  fd = open(fname, O_WRONLY|O_CREAT);
  if (fd < 0) return 1;

  /* file mode */
  if (fchmod(fd, mode)) {
    perror("aux_luatar_new");
    /* FIXME fail ? */
  }

  /* set compression */
  if (compression > 9) compression = 9;
  else if (compression < 9) compression = 0;
  zmode[2] = compression + '0';

  /* prepare opening file */
  switch (type) {
  case LUATAR_TYPE_PLAIN:
    break;
#ifdef USE_GZIP
  case LUATAR_TYPE_ZIP:
  case LUATAR_TYPE_GZIP: 
    if (!(F = gzdopen(fd, zmode))) return 1;
    ttype = &c_gzip;
    break;
#endif
#ifdef USE_BZIP2
  case LUATAR_TYPE_BZIP2:
    if (!(F = BZ2_bzdopen(fd, zmode))) return 1;
    ttype = &c_bzip2;
    break;
#endif
  default:
    errno = ENOSYS;
    return 1;
  }

  /* add stream<->fd mapping */
  if (F)
    luatar_cstream_add(fd, F);

  /* adjust ud */
  ud->read_only   = 0;
  ud->stream_type = type;
  ud->status      = LUATAR_STATUS_INIT;

  /* open tar */
  if (
      tar_fdopen(&(ud->file), fd, /*FIXME why is this (char *) ????*/ (char *) fname, ttype, O_WRONLY, 0, flags)
      ) {
    ud->status = LUATAR_STATUS_ERROR;
    return 1;
  }

  return 0;
}

int aux_luatar_close(tar_ud *ud)
{
  if (ud->file) {
    tar_close(ud->file);
    ud->file = NULL;
  }

  return 0;
}
